@@ -13,13 +13,13 @@ Data::GUID - globally unique identifiers
 
 =head1 VERSION
 
-version 0.02
+version 0.04
 
- $Id: /my/cs/projects/guid/trunk/lib/Data/GUID.pm 19227 2006-02-28T01:06:16.070217Z rjbs  $
+ $Id: /my/cs/projects/guid/trunk/lib/Data/GUID.pm 19690 2006-03-12T02:22:09.139548Z rjbs  $
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.04';
 
 =head1 SYNOPSIS
 
@@ -92,14 +92,18 @@ sub from_data_uuid {
   bless \$value => $class;
 }
 
-my $hex    = qr/[0-9A-F]/i;
-my $base64 = qr{[A-Z0-9+/=]}i;
+my ($hex, $base64, %type);
 
-my %type = ( # uuid_method  validation_regex
-  string => [ 'string',     qr/\A$hex{8}-(?:$hex{4}-){3}$hex{12}\z/, ],
-  hex    => [ 'hexstring',  qr/\A0x$hex{32}\z/,                      ],
-  base64 => [ 'b64string',  qr/\A$base64{24}\z/,                     ],
-);
+BEGIN { # because %type must be populated for method/exporter generation
+  $hex    = qr/[0-9A-F]/i;
+  $base64 = qr{[A-Z0-9+/=]}i;
+
+  %type = ( # uuid_method  validation_regex
+    string => [ 'string',     qr/\A$hex{8}-(?:$hex{4}-){3}$hex{12}\z/, ],
+    hex    => [ 'hexstring',  qr/\A0x$hex{32}\z/,                      ],
+    base64 => [ 'b64string',  qr/\A$base64{24}\z/,                     ],
+  );
+}
 
 # provided for test scripts
 sub __type_regex { shift; $type{$_[0]}[1] }
@@ -131,27 +135,74 @@ sub _install_as_method {
   Sub::Install::install_sub({ code => $our_to_method, as => "as_$type" });
 }
 
-do {
-  while (my ($type, $profile) = each %type) {
-    _install_from_method($type, @$profile);
-    _install_as_method  ($type, @$profile);
+BEGIN { # possibly unnecessary -- rjbs, 2006-03-11
+  do {
+    while (my ($type, $profile) = each %type) {
+      _install_from_method($type, @$profile);
+      _install_as_method  ($type, @$profile);
+    }
+  };
+}
+
+sub _from_multitype {
+  my ($class, $what, $types) = @_;
+  sub {
+    my ($class, $value) = @_;
+    return $value if eval { $value->isa('Data::GUID') };
+
+    my $value_string = defined $value ? qq{"$value"} : 'undef';
+
+    # The only good ref is a blessed ref, and only into our denomination!
+    if (my $ref = ref $value) {
+      Carp::croak "a $ref reference is not a valid GUID $what"
+    }
+    
+    for my $type (@$types) {
+      my $from = "from_$type";
+      my $guid = eval { $class->$from($value); };
+      return $guid if $guid;
+    }
+
+    Carp::croak "$value_string is not a valid GUID $what";
   }
-};
+}
 
-sub _GUID {
-  my ($class, $value) = @_;
-  return $value if eval { $value->isa('Data::GUID') };
+=head2 C< from_any_string >
 
-  # The only good ref is a blessed ref, and only into our denomination!
-  return if (ref $value);
-  
-  for my $type ((keys %type), 'data_uuid') {
-    my $from = "from_$type";
-    my $guid = eval { $class->$from($value); };
-    return $guid if $guid;
-  }
+  my $string = get_string_from_ether;
 
-  return;
+  my $guid = Data::GUID->from_any_string($string);
+
+This method returns a Data::GUID object for the given string, trying all known
+string interpretations.  An exception is thrown if the value is not a valid
+GUID string.
+
+=cut
+
+BEGIN { # possibly unnecessary -- rjbs, 2006-03-11
+  Sub::Install::install_sub({
+    code => __PACKAGE__->_from_multitype('string', [ keys %type ]),
+    as   => 'from_any_string',
+  });
+}
+
+=head2 C< best_guess >
+
+  my $value = get_value_from_ether;
+
+  my $guid = Data::GUID->best_guess($value);
+
+This method returns a Data::GUID object for the given value, trying everything
+it can.  It works like C<L</from_any_string>>, but will also accept Data::UUID
+values.  (In effect, this means that any sixteen byte value is acceptable.)
+
+=cut
+
+BEGIN { # possibly unnecessary -- rjbs, 2006-03-11
+  Sub::Install::install_sub({
+    code => __PACKAGE__->_from_multitype('value', [(keys %type), 'data_uuid']),
+    as   => 'best_guess',
+  });
 }
 
 =head1 GUIDS INTO STRINGS
@@ -220,19 +271,21 @@ use overload
 
 =head1 IMPORTING
 
-Data::GUID does not export any subroutines by default, but it provides four
+Data::GUID does not export any subroutines by default, but it provides a few
 routines which will be imported on request.  These routines may be called as
-class methods, or may be imported to be called as subroutines.
+class methods, or may be imported to be called as subroutines.  Calling them by
+fully qualified name is incorrect.
+
+  use Data::GUID qw(guid);
+
+  my $guid = guid;             # OK
+  my $guid = Data::GUID->guid; # OK
+  my $guid = Data::GUID::guid; # NOT OK
 
 =cut
 
 =head2 C< guid >
 
-  use Data::GUID qw(guid);
-
-  my $guid_1 = Data::GUID->guid;
-  my $guid_2 = guid;
-
 This routine returns a new Data::GUID object.
 
 =head2 C< guid_string >
@@ -247,44 +300,46 @@ This returns the hex representation of a new GUID.
 
 This returns the base64 representation of a new GUID.
 
+=head2 C< guid_from_anything >
+
+This returns the result of calling the C<L</from_any_string>> method.
+
 =cut
 
-{ no warnings 'once'; *guid = \&new; }
+BEGIN {
+  Sub::Install::install_sub({ code => 'new', as => 'guid' });
 
-for my $type (keys %type) {
-  my $method = "guid_$type";
-  my $as     = "as_$type";
+  for my $type (keys %type) {
+    my $method = "guid_$type";
+    my $as     = "as_$type";
 
-  no strict 'refs';
-  *$method = sub {
-    my ($class) = @_;
-    $class->new->$as;
+    no strict 'refs';
+    *$method = sub {
+      my ($class) = @_;
+      $class->new->$as;
+    }
   }
 }
 
-my %exports = map { $_ => 1 } ('guid', map { "guid_$_" } keys %type);
-
-$exports{_GUID} = 1;
-
-sub import {
-  my ($class, @to_export) = @_;
-  my $into = caller(0);
-  @to_export = keys %exports if grep { $_ eq ':all' } @to_export;
-  
-  for my $sub (@to_export) {
-    Carp::croak qq{"$sub" is not exported by the Data::GUID module}
-      unless $exports{ $sub };
-    Sub::Install::install_sub({
-      code => sub { $class->$sub },
-      into => $into,
-      as   => $sub,
-    });
-  }
+sub _curry_class {
+  my ($class, $subname, $eval) = @_;
+  return $eval ? sub { eval { $class->$subname(@_) } }
+               : sub { $class->$subname(@_) };
 }
 
-=head1 AUTHOR
+my %exports;
+BEGIN {
+  %exports
+    = map { my $method = $_; $_ => sub { _curry_class($_[0], $method) } } 
+    ((map { "guid_$_" } keys %type), 'guid');
+}
 
-Ricardo SIGNES, C<< <rjbs@cpan.org> >>
+use Sub::Exporter -setup => {
+  exports => {
+    %exports, # defined just above
+    guid_from_anything => sub { _curry_class($_[0], 'from_any_string', 1) },
+  }
+};
 
 =head1 TODO
 
@@ -298,6 +353,10 @@ Ricardo SIGNES, C<< <rjbs@cpan.org> >>
 
 =back
 
+=head1 AUTHOR
+
+Ricardo SIGNES, C<< <rjbs@cpan.org> >>
+
 =head1 BUGS
 
 Please report any bugs or feature requests to
